////////////////////////////////////////////////////////////////////////
// A very basic benchmark of the compile- and link-time costs of adding
// s11n-related type registrations.
//
// Author: stephan@s11n.net
// License: Do As You Damned Well Please
////////////////////////////////////////////////////////////////////////

#include <stdexcept>


////////////////////////////////////////////////////////////////////////
#include <s11n.net/s11n/s11nlite.hpp> // s11n & s11nlite frameworks
////////////////////////////////////////////////////////////////////////


#define CONTAINER_TYPE_COUNT 4

/***
    Below is are timings for compiling and linking this file on my box (1.4GHz
    PC w/ standard RAM and HDD), each build run several times, low and
    high times shown. Each increment of COUNTAINER_TYPE_COUNT adds one
    new standard container, plus possibly one POD:

    CONTAINER_TYPE_COUNT 0 == 1.21s - 1.27s
    w/o linking: 1.03s

    1 == 3.05s - 3.16s (+pod_double proxy)
    w/o linking: 2.74

    2 == 3.81s - 3.99s
    w/o linking: 3.39s

    3 == 5.38s - 5.57s  (+pod_string proxy)
    w/o linking: 4.87s

    4 == 6.02s - 6.25s
    w/o linking: 5.39s

    Approximately 0.03s of each time is build-process overhead.

    Works out to about 0.5-0.9ish seconds per new type.

    This is no particular reason why registration of types other than
    the standard containers would take a different amount of time. The
    containers are used here because those proxies ship with the
    library and i use them often in my client code.

*/

#if (CONTAINER_TYPE_COUNT > 0)
#include <s11n.net/s11n/proxy/pod/double.hpp>
#include <s11n.net/s11n/proxy/std/valarray.hpp>
#endif

#if (CONTAINER_TYPE_COUNT > 1)
#include <s11n.net/s11n/proxy/std/list.hpp>
#endif

#if (CONTAINER_TYPE_COUNT > 2)
#include <s11n.net/s11n/proxy/pod/string.hpp>
#include <s11n.net/s11n/proxy/std/map.hpp>
#endif

#if (CONTAINER_TYPE_COUNT > 3)
#include <s11n.net/s11n/proxy/std/vector.hpp>
#endif


void test_buildspeed()
{
#if (CONTAINER_TYPE_COUNT > 0)
        typedef std::valarray<double> VA;
	VA va;
        s11nlite::save( va, std::cout );
#endif

#if (CONTAINER_TYPE_COUNT > 1)
        typedef std::list<double> LT;
	LT lt;
        s11nlite::save( lt, std::cout );
#endif

#if (CONTAINER_TYPE_COUNT > 2)
        typedef std::map<double,std::string> MT;
	MT mt;
        s11nlite::save( mt, std::cout );
#endif

#if (CONTAINER_TYPE_COUNT > 3)
        typedef std::vector<double> VT;
	VT vt;
        s11nlite::save( vt, std::cout );
#endif


}

int
main( int argc, char **argv )
{
 	using namespace s11n::debug;
 	trace_mask( trace_mask() | TRACE_INFO | TRACE_TRIVIAL );
        s11nlite::serializer_class( "simplexml" );

	try
	{
		test_buildspeed();
	}
	catch( const std::exception & ex )
	{
		CERR << "EXCEPTION: " << ex.what() << "\n";
		return 1;
	}
        return 0;
}
