////////////////////////////////////////////////////////////////////////
// this is the main app for the libs11n unit tests.

#include "UnitTest.hpp"
#include <memory>

bool UnitTest::run( std::string const & name )
{
	UT_COUT << "Running test: " << name << std::endl;
	UnitFactory & f = UnitFactory::instance();
	if( ! f.provides(name) )
	{
		std::string dll = ::s11n::plugin::open(name);
		std::string err( ::s11n::plugin::dll_error() );
		if( dll.empty() )
		{
			UT_CERR << "Warning: open("<<name<<") failed: " << err << '\n';
		}
		if( ! f.provides(name) )
		{
			throw std::runtime_error("UnitFactory doesn't provide a test named '"+name+"'.");
		}
	}
	std::auto_ptr<UnitTest> t( f.create( name ) );
	if( ! t.get() )
	{
		throw std::runtime_error("UnitFactory.create("+name+") failed!");
	}
	return t->run();
}

NoopTest::NoopTest() : UnitTest("NoopTest") {}
NoopTest::~NoopTest() {}
bool NoopTest::run()
{
	UT_COUT << "Test " << this->className() << " does nothing.\n";
	return true;
}
#define UNIT_TEST NoopTest
#define UNIT_TEST_NAME "NoopTest"
#include "RegisterUnitTest.hpp"


int main( int argc, char **argv )
{
	if( 1 == argc )
	{
		UT_CERR << "Usage: " << argv[0] << " TestName1 [... TestNameN]" << std::endl;
		return 127;
	}
	try
	{
		std::string tn;
		for( int i = 1; i < argc; ++i )
		{
			tn = argv[i];
			if( ! UnitTest::run( tn ) )
			{
				throw std::runtime_error( "Test failed: "+tn );
			}
		}
	}
	catch( std::exception const & ex )
	{
		UT_CERR << "EXCEPTION: " << ex.what() << std::endl;
		return 1;
	}
	catch(...)
	{
		UT_CERR << "UNKNOWN EXCEPTION!" << std::endl;
		return 2;

	}
	return 0;
}
